/*
 * stable.h
 * This file is part of qps -- Qt-based visual process status monitor
 *
 * Copyright 2014 dae hyun, yang <daehyun.yang@gmail.com>
 * Copyright 2015 Paulo Lieuthier <paulolieuthier@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */


// for Precompiled Header !
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if defined __cplusplus

#include <QBitArray>
#include <QHash>
#include <QList>
#include <QRect>
#include <QString>
#include <QStringList>
#include <QProcess>
#include <QSettings>

#include <QItemDelegate>
#include <QStandardItemModel>

#include <QShortcut>
#include <QAction>

#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QGridLayout>

#include <QColor>
#include <QFont>

#include <QWidget>
#include <QLabel>
#include <QFrame>
#include <QPixmap>
#include <QDialog>
#include <QSlider>
#include <QSpinBox>
#include <QTabWidget>

#include <QStyleOption>
#include <QAbstractScrollArea>

#include <QPainter>
#include <QScrollBar>

#include <QAbstractButton>
#include <QPushButton>
#include <QCheckBox>
#include <QRadioButton>
#include <QStatusBar>
#include <QToolButton>
#include <QGroupBox>
#include <QMessageBox>

#include <QToolTip>
#include <QLineEdit>
#include <QComboBox>
#include <QTextEdit>

#include <QTimeLine>
#include <QDir>

#include <QMenu>
#include <QMenuBar>

#include <QListView>
#include <QHeaderView>
#include <QTreeView>

#include <QTextBrowser>
#include <QUrl>

#include <QClipboard>
#include <QApplication>
#include <QSessionManager>

#include <QPaintEvent>
#include <QMouseEvent>
#include <QCloseEvent>
#include <QTimerEvent>
#include <QKeyEvent>
#include <QResizeEvent>

#include <QFontComboBox>

#include "svec.h"
#endif
